package com.supervision.ai.service.hub.service;

import cn.hutool.core.bean.BeanPath;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.supervision.ai.service.hub.constant.ComfyuiParamTypeEnum;
import com.supervision.ai.service.hub.constant.ComfyuiRunStatusEnum;
import com.supervision.ai.service.hub.domain.ComfyuiTaskParam;
import com.supervision.ai.service.hub.dto.comfyui.*;
import com.supervision.ai.service.hub.dto.sys.FileDTO;
import com.supervision.ai.service.hub.service.impl.ComfyUIApiServiceImpl;
import lombok.extern.slf4j.Slf4j;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.stream.Collectors;

@Slf4j
public class ComfyUITaskScheduler {

    private final NodeManager nodeManager;
    private final SchedulingStrategy strategy;

    private final Map<String, ComfyUIApiService> comfyUIApiServiceMap = new ConcurrentHashMap<>();

    private  ComfyUIService comfyUIService;

    private ComfyuiTaskService comfyUITaskService;

    private SysFileService sysFileService;


    public ComfyUITaskScheduler(NodeManager nodeManager, SchedulingStrategy strategy) {
        this.nodeManager = nodeManager;
        this.strategy = strategy;
    }

    /**
     * 执行任务
     */
    void executeTask() {
        TaskDTO task = strategy.selectNextTask();
        if (task == null) {
            log.info("executeTask:没有获取到可用的任务....");
            return;
        }

        while (true) {
            ServerNode node = nodeManager.selectAvailableNode();
            if (null == node) {
                log.error("executeTask:没有获取到可用的节点....");
                return;
            }
            // 运行提示词
            try {
                boolean success = runPrompt(node, task);
                if (success) {
                    log.info("executeTask:任务:{}执行成功", task.getId());
                    break;
                } else {
                    // 失败后重新选择节点
                    node = nodeManager.selectAvailableNode();
                    if (null == node) {
                        log.error("executeTask:没有获取到可用的节点....");
                        break;
                    }
                }
            } catch (Exception e) {
                //todo: 如果是链接异常设置节点
                throw new RuntimeException(e);
            }

        }
    }


    private boolean runPrompt(ServerNode node,TaskDTO task){
        ComfyUIApiService comfyUIApiService = this.getComfyUIApiService(node);

        // 设置参数内容
        JSONObject prompt = task.getPrompt();
        for (ComfyuiTaskParam param : task.getParams()) {
            if (StrUtil.equals("4",param.getComponentType())){
                // 上传图片
                FileDTO fileDTO = sysFileService.queryById(param.getValue());
                byte[] bytes = sysFileService.downloadFile(param.getValue());
                ImageUploadDTO imageUploadDTO = comfyUIApiService.uploadImage(bytes, param.getValue() + "." + fileDTO.getFileType());
                param.setValue(imageUploadDTO.getName());
            }
            ComfyuiParamTypeEnum comfyuiParamTypeEnum = ComfyuiParamTypeEnum.getByCode(Integer.parseInt(param.getParamType()));
            Object value = comfyuiParamTypeEnum.coverType(param.getValue());
            log.info("设置参数key：{},参数value:{}", param.getParamKey(), value);
            BeanPath.create(param.getParamKey()).set(prompt, value);
        }
        // 运行提示词
        PromptRunDTO promptRunDTO = comfyUIApiService.runPrompt(prompt);
        Map<String, Object> error = promptRunDTO.getError();

        if (CollUtil.isNotEmpty(error)){
            // 执行失败
            String errorMessage = (String) error.get("message");
            log.error("executeTask:任务:{}执行失败，错误信息：{}",task.getId(),errorMessage);
            nodeManager.markNodeFailed(node.getId());
            return false;
        }else {
            // 任务提交成功
            nodeManager.recoverNode(node.getId());
            node.addRunningTask(task);
            task.setComfyuiServer(node.getHost()+":"+node.getPort());
            task.setRunStatus(ComfyuiRunStatusEnum.RUNNING.getStatus());
            task.setRunStartTime(LocalDateTime.now());
            comfyUITaskService.updateTaskResult(task);
            return true;
        }
    }

    /**
     * 查询任务结果
     */
    void queryTaskResult() {
        List<ServerNode> serverNodes = nodeManager.listAllNodes();
        for (ServerNode serverNode : serverNodes) {
            CopyOnWriteArrayList<TaskDTO> runningTasks = serverNode.getRunningTasks();
            for (TaskDTO taskDTO : runningTasks) {
                ComfyUIApiService comfyUIApiService = this.getComfyUIApiService(serverNode);
                comfyUIApiService.asyncQueryHistory(taskDTO.getComfyuiTaskId(), result -> {
                    Map<String,Object> taskResult = JSONUtil.toBean(result, HashMap.class);
                    Map<String,Object> thisResult = (Map<String, Object>) taskResult.get(taskDTO.getComfyuiTaskId());
                    boolean finish = comfyUIService.taskIsCompleted(thisResult);
                    if (finish){
                        // 移除运行中的任务
                        serverNode.removeRunningTask(taskDTO);
                        List<String> imageList = comfyUIService.getImageList(thisResult);
                        List<ComfyUITaskResult> comfyUITaskResults = imageList.stream().map(imageId -> {
                            ComfyUITaskResult comfyUITaskResult = new ComfyUITaskResult();
                            comfyUITaskResult.setTaskId(taskDTO.getId());
                            comfyUITaskResult.setFileName(imageId);
                            return comfyUITaskResult;
                        }).collect(Collectors.toList());
                        taskDTO.setResults(comfyUITaskResults);
                        taskDTO.setRunStatus(ComfyuiRunStatusEnum.SUCCESS.getStatus());
                        taskDTO.setRunEndTime(LocalDateTime.now());
                        comfyUITaskService.updateTaskResult(taskDTO);
                        comfyUITaskService.notifyWebhook(taskDTO);
                    }
                });
            }
        }
    }

    private ComfyUIApiService getComfyUIApiService(ServerNode node) {
        return comfyUIApiServiceMap.computeIfAbsent(node.getId(),
                id -> new ComfyUIApiServiceImpl(node.getHost(), String.valueOf(node.getPort())));
    }
}
