package com.supervision.ai.service.hub.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.supervision.ai.service.hub.dto.comfyui.ImageUploadDTO;
import com.supervision.ai.service.hub.dto.comfyui.PromptRunDTO;
import com.supervision.ai.service.hub.dto.comfyui.ResourceViewDTO;
import com.supervision.ai.service.hub.service.ComfyUIApiService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import java.net.URI;
import java.net.http.HttpClient;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.function.Consumer;

@Slf4j
@Service
public class ComfyUIApiServiceImpl implements ComfyUIApiService {

    @Value("${comfyUI.server.ip}")
    private String comfyUIServiceIp;

    @Value("${comfyUI.server.port}")
    private String comfyUIServicePort;

    private static final ObjectMapper objectMapper = new ObjectMapper();

    public ComfyUIApiServiceImpl() {
    }

    public ComfyUIApiServiceImpl(String comfyUIServiceIp, String comfyUIServicePort) {
        this.comfyUIServiceIp = comfyUIServiceIp;
        this.comfyUIServicePort = comfyUIServicePort;
    }

    @Override
    public PromptRunDTO runPrompt(JSONObject prompt) {
        String url = "http://" + comfyUIServiceIp + ":" + comfyUIServicePort + "/prompt";
        log.info("comfyUI runPrompt 请求地址：{}",url);
        HttpRequest request = HttpUtil.createPost(url).body(prompt.toJSONString());
        try (HttpResponse execute = request.execute()){
            String body = execute.body();
            log.info("comfyUI runPrompt 请求结果：{}",body);
            if (execute.isOk()){
                try {
                    return objectMapper.readValue(body, PromptRunDTO.class);
                } catch (JsonProcessingException e) {
                    throw new RuntimeException(e);
                }
            }else {
                throw new RuntimeException("/prompt请求失败");
            }
        }
    }

    @Override
    public Map<String, Object> queryHistory(String promptId) {

        String url = "http://" + comfyUIServiceIp + ":" + comfyUIServicePort + "/history" + (StrUtil.isEmpty(promptId)? "" : "/"+promptId);
        log.info("comfyUI queryHistory 请求地址：{}",url);
        HttpRequest header = HttpUtil.createGet(url).header("Accept", "application/json");
        HttpResponse response = header.executeAsync();
        try (HttpResponse execute = HttpUtil.createGet(url).execute()){
            if (execute.isOk()){
                String body = execute.body();
                try {
                    return objectMapper.readValue(body, Map.class);
                } catch (JsonProcessingException e) {
                    throw new RuntimeException(e);
                }
            }else {
                throw new RuntimeException("/history请求失败");
            }
        }
    }

    @Override
    public void asyncQueryHistory(String promptId, Consumer<String> callback) {

        String url = "http://" + comfyUIServiceIp + ":" + comfyUIServicePort + "/history" + (StrUtil.isEmpty(promptId)? "" : "/"+promptId);
        java.net.http.HttpClient client = HttpClient.newHttpClient();
        java.net.http.HttpRequest request = java.net.http.HttpRequest.newBuilder()
                .uri(URI.create(url))
                .build();
        CompletableFuture<java.net.http.HttpResponse<String>> future =  client.sendAsync(request, java.net.http.HttpResponse.BodyHandlers.ofString());

        future.thenApply(java.net.http.HttpResponse::body)
                .thenAccept(callback)
                .exceptionally(ex -> {
                    log.error("asyncQueryHistory:发送请求失败", ex);
                    return null;
                });
    }

    @Override
    public ImageUploadDTO uploadImage(byte[] file, String fileName) {
        String url = "http://" + comfyUIServiceIp + ":" + comfyUIServicePort + "/upload/image";
        HttpRequest request = HttpUtil.createPost(url);
        request.form("image",file,fileName);
        try (HttpResponse execute = request.execute()){
            if (execute.isOk()){
                String body = execute.body();
                log.info("comfyUI uploadImage 请求结果：{}",body);
                try {
                    return objectMapper.readValue(body, ImageUploadDTO.class);
                } catch (JsonProcessingException e) {
                    throw new RuntimeException(e);
                }
            }else {
                throw new RuntimeException("/upload/image请求失败");
            }
        }
    }

    @Override
    public ResourceViewDTO downloadView(String imageId) {
        Assert.notEmpty(imageId,"imageId不能为空");
        String url = "http://" + comfyUIServiceIp + ":" + comfyUIServicePort + "/view?filename=" + imageId;
        log.info("comfyUI downloadImage 请求地址：{}",url);
        try (HttpResponse execute = HttpUtil.createGet(url).execute()){
            if (execute.isOk()){
                return new ResourceViewDTO(execute.getFileNameFromDisposition("filename"),
                        execute.header("content-type"), execute.bodyBytes());
            }else {
                throw new RuntimeException("下载图片请求失败");
            }
        }
    }
}
