package com.supervision.ai.service.hub.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.supervision.ai.service.hub.dto.comfyui.ServerNode;
import com.supervision.ai.service.hub.service.NodeManager;
import lombok.extern.slf4j.Slf4j;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

@Slf4j
public class ComfyUINodeManager implements NodeManager {

    private List<ServerNode> serverNodes = new ArrayList<>();

    private int retryTimeThread = 60;
    @Override
    public ServerNode selectAvailableNode() {
        // 优先选择空闲且没有失败次数的节点
        // 如果所有的节点都没有失败次数，则选择正在运行次数最小的节点（本次规则中选择运行次数为0的节点）
        // 如果所有的节点都失效，则选择一个静默时间最长的节点
        ServerNode selectedNode = null;
        for (ServerNode node : serverNodes) {
            log.info("开始查找存活且空闲的节点....");
            if (node.getRunningTaskCount() == 0 && node.getFailureCount() == 0) {
                selectedNode = node;
                log.info("查找到存活且空闲的节点：host:{}:{}....",node.getId(),node.getPort());
                node.setLastSelectedTime(LocalDateTime.now());
                break;
            }
        }
        if (null == selectedNode){
            log.warn("没有查找到存活且空闲的节点，开始查找已经离线的节点....");
            List<ServerNode> failServerNode = new ArrayList<>();
            for (ServerNode node : serverNodes) {
                if (node.getFailureCount() != 0) {
                    failServerNode.add(node);
                }
            }
            log.info("所有离线节点个数为：{}",failServerNode.size());
            failServerNode.sort(Comparator.comparing(ServerNode::getLastSelectedTime));
            ServerNode last = CollUtil.getLast(failServerNode);
            // 给失败的节点一次机会
            if (last.getLastSelectedTime().plusSeconds(retryTimeThread).isBefore(LocalDateTime.now())) {
                log.info("选择失败的节点：host:{}:{},上次检查时间是:{}....",last.getId(),last.getPort(),last.getLastSelectedTime());
                selectedNode = last;
                selectedNode.setLastSelectedTime(LocalDateTime.now());
            }
        }
        return selectedNode;
    }

    @Override
    public void markNodeFailed(String nodeId) {
        for (ServerNode node : serverNodes) {
            if (node.getId().equals(nodeId)) {
                node.incrementFailureCount();
                break;
            }
        }
    }

    @Override
    public void recoverNode(String nodeId) {
        for (ServerNode node : serverNodes) {
            if (node.getId().equals(nodeId)) {
                node.resetFailureCount();
                break;
            }
        }
    }

    @Override
    public List<ServerNode> listAllNodes() {
        return this.serverNodes;
    }

    @Override
    public void addNode(ServerNode node) {
        // 检查节点是否已经存在
        for (ServerNode serverNode : serverNodes) {
            if (StrUtil.equals(serverNode.getHost(), node.getHost()) && serverNode.getPort() == node.getPort()) {
                // 节点已经存在
                log.info("节点已经存在，host:{},port:{}", node.getHost(), node.getPort());
                return;
            }
        }
        // 添加节点
        serverNodes.add(node);
        log.info("添加节点成功，id:{},host:{},port:{}", node.getId(), node.getHost(), node.getPort());
    }
}
